#ifndef BINARY_TREE_H
#define BINARY_TREE_H 1

/**
 * Defines a binary tree to store integer values.
 */
struct _Tree;

/**
 * Defines one node of a binary tree.
 */
struct _Node;

typedef struct _Tree Tree;
typedef struct _Node Node;

enum ORDER { DEPTH, BREADTH, INORDER };

//! Init. binary tree.
/**
 * Initialize an empty binary tree.
 * \return pointer to the created binary tree
 */
Tree* initTree( void );

//! Tree height
/**
 * Computes the high of a binary tree.
 * \param tree the binary tree
 * \return     the binary trees high
 */
size_t treeHeight( const Tree* tree );

//! Number of nodes
/**
 * Counts the nodes in a binary tree.
 * \param tree the binary tree
 * \return     the number of the tree nodes
 */
size_t treeSize( const Tree* tree );

//! Search for specific value
/**
 * Search for a certain value in a binary tree.
 * \param tree  the binary tree
 * \param value the value to search for
 * \return      the node that contains the value, otherwise NULL
 */
Node* search( const Tree* root, const int value );

//! Add a new value
/**
 * Inserts an integer value to the binary tree. Duplicates are not allowed.
 * \param tree  the binary tree
 * \param value the value to insert
 */
void insert( Tree* tree, const int value );

//! erase a value
/**
 * Erases a single value from the binary tree, if it contains it.
 * \param tree  the binary tree
 * \param value the value that has to be erase from the binary tree
 */
void erase( Tree* tree, const int value );

//! Clear binary tree content
/**
 * Delete all nodes of a binary tree and the tree itself.
 * \param tree the binary tree
 */
void deleteTree( Tree* tree );

//! Print tree to console
/**
 * Print all values stored in a binary tree (depth-first, breadth-first or in-order).
 * \param tree  the binary tree
 * \param order pass DEPTH for depth-first, BREADTH for breadth-first or INORDER for in-order; otherwise no value will be printed to console
 */
void printTree( const Tree* tree, const enum ORDER order );

#endif /* BINARY_TREE_H */
