#include <stdio.h>
#include <stdlib.h>

#include "queue.h"

typedef struct _QElement
{
    Node* value;
    struct _QElement* next;
} QElement;


struct _Queue
{
    QElement* start;
};


Queue* initQueue( void )
{
    Queue* queue = (Queue*) malloc( sizeof( Queue ) );
    if( queue == NULL )
    {
        fprintf( stderr, "ERROR: There is not enough free memory available!\n" );
        exit( EXIT_FAILURE );
    }

    queue->start = NULL; //(*queue).start = NULL;
    return queue;
}


QElement* initQElement( void )
{
    QElement* qe = (QElement*) malloc( sizeof( QElement ) );
    if( qe == NULL )
    {
        fprintf( stderr, "ERROR: There is not enough free memory available!\n" );
        exit( EXIT_FAILURE );
    }

    qe->value = NULL;
    qe->next = NULL;
    return qe;
}


void push( Queue* queue, Node* value )
{
    // Was 'queue' init.?
    if( queue == NULL )
    {
        return;
    }

    // are there already values in the queue -> append the new one
    if( queue->start != NULL )
    {
        // search for last element in the queue
        QElement* last = queue->start;
        while( last->next != NULL )
        {
            last = last->next;
        }

        QElement* qe = initQElement();
        if( qe == NULL )
        {
            fprintf( stderr, "ERROR: There is not enough free memory available!\n" );
            exit( EXIT_FAILURE );
        }

        qe->value = value;
        qe->next = NULL;
        last->next = qe;
    }
    else // else: init. first element of queue
    {
        queue->start = initQElement();
        queue->start->value = value;
        queue->start->next = NULL;
    }
}


Node* pop( Queue* queue )
{
    Node* tmp = NULL; // init. default return value

    if( queue != NULL && queue->start != NULL )
    {
        // get node encapsulated by the QElement
        tmp = queue->start->value;
        // free resource used for the QElement and upadte queues 'start' element
        QElement* tmpQE = queue->start->next;
        free( queue->start );
        queue->start = tmpQE;
    }

    return tmp;
}


size_t queueSize( const Queue* queue )
{
    size_t size = 0;

    QElement* qe = queue->start;
    while( qe != NULL )
    {
        size++;
        qe = qe->next;
    }

    return size;
}


void deleteQueue( Queue* queue )
{
    if( queue == NULL )
    {
        return;
    }

    QElement* qe = queue->start;
    while( qe != NULL )
    {
        free( qe );
        qe = qe->next;
    }

    free( queue );
}
